package fr.velossity.twitter.impl;

import org.osgi.service.log.LogService;

import twitter4j.DirectMessage;
import twitter4j.Twitter;
import twitter4j.TwitterException;
import twitter4j.TwitterFactory;
import twitter4j.conf.ConfigurationBuilder;
import fr.velossity.twitter.Recipient;


/**
 * Transforms the Twitter service so as we can only and very
 * easily send direct messages. For that, we have defined a
 * Twitter application named OSGi training. 
 * @author C. Saint-Marcel
 * 
 */
public class TwitterAdapter implements Recipient {
	
	/**
	 * iPOJO field referencing the logger service.
	 */
	LogService logger;
	
	/**
	 * iPOJO property.
	 * Twitter account name used to define the recipient of the direct message.
	 */
	String user;

	/**
	 * iPOJO property.
	 * Twitter account name used to define the sender of the direct message.
	 */
	String sender;

	/**
	 * iPOJO property.
	 * Application consumer key.
	 */
	String consumerKey;

	/**
	 * iPOJO property.
	 * Application consumer secret.
	 */
	String consumerSecret;

	/**
	 * iPOJO property.
	 * Application access token.
	 */
	String accessToken;

	/**
	 * iPOJO property.
	 * Application access token secret.
	 */
	String accessTokenSecret;			

	/**
	 * iPOJO property.
	 * Optional http proxy host.
	 */
	String httpProxyHost;

	/**
	 * iPOJO property.
	 * Optional http proxy port.
	 */
	int httpProxyPort;

	/**
	 * iPOJO property.
	 * Optional http proxy user.
	 */
	String httpProxyUser;

	/**
	 * iPOJO property.
	 * Optional http proxy password.
	 */
	String httpProxyPassword;
	
	/**
	 * The twitter adaptee.
	 */
	private Twitter twitter;
	

	/**
	 * iPOJO callback, called at activation.
	 */
	public void activate() {
		// Twitter configuration
		ConfigurationBuilder cb = new ConfigurationBuilder();
		cb.setDebugEnabled(false)
		  .setOAuthConsumerKey(consumerKey)
		  .setOAuthConsumerSecret(consumerSecret)
		  .setOAuthAccessToken(accessToken)
		  .setOAuthAccessTokenSecret(accessTokenSecret);
		
		// Proxy optional configuration
		if (httpProxyHost != null) {
		  cb.setHttpProxyHost(httpProxyHost)
		  	.setHttpProxyPort(httpProxyPort)
		  	.setHttpProxyUser(httpProxyUser)
		  	.setHttpProxyPassword(httpProxyPassword);
		}
		
		TwitterFactory tf = new TwitterFactory(cb.build());
		// Get the instance
		twitter = tf.getInstance();
		
		// Just to signal the service instanciation
		sendMessage("Twitter service activated");
	}

	/**
	 * iPOJO callback, called at inactivation.
	 */
	public void inactivate() {
		logger.log(LogService.LOG_INFO, "Bundle deactivated");
	}

	/**
	 * Delegates the message sending to the twitter service.
	 */
	public boolean sendMessage(String message) {
		boolean result = true;
		try {
			// Sending Direct Messages in message box
			DirectMessage tMessage = twitter.sendDirectMessage(user, "@" + sender + " " + message);
			logger.log(LogService.LOG_INFO, "Message sent to @" + tMessage.getRecipientScreenName() + "::" + tMessage.getText());
		} catch (TwitterException e) {
			logger.log(LogService.LOG_ERROR, "Unable to send a tweet", e);
			result = false;
		}
		return result;
	}
}
